#!D:\bld\gdal-split_1652654803939\_h_env\python.exe
# EASY-INSTALL-SCRIPT: 'GDAL==3.5.0','gdal_merge.py'
__requires__ = 'GDAL==3.5.0'
__import__('pkg_resources').run_script('GDAL==3.5.0', 'gdal_merge.py')
